/*
 File/Sketch Name: Sound_to_Light

 Version No.: v1.0 Created 19 April, 2022
 
 Original Author: Spencer Johnson, Purdue Polytechnic Institute MET

Contains code from: 
  Clyde A Lettsome, PhD, PE, MEM
  File/Sketch Name: AudioFrequncyDetector
  v1.0 Created 12 December, 2019
 
 Description:  This code/sketch makes displays the approximate frequency of the loudest sound detected by a sound detection module. For this project, the analog output from the 
 sound module detector sends the analog audio signal detected to A0 of the Arduino Mega2560. The analog signal is sampled and quantized (digitized). A Fast Fourier Transform (FFT) is
 then performed on the digitized data. The FFT converts the digital data from the approximate discrete-time domain result. The maximum frequency of the approximate discrete-time
 domain result is then determined and displayed via the Arduino IDE Serial Monitor. The output of the frequency detection is then fed into if statements to determine the frequenccy range 
 in 50 HZ increments from 100 Hz to 1000 Hz. This is used to assign a value between 0 and 17 to be fed into switch case construct to enter a for loop to address all LED's to predetermined
 RGB color values. The program will repeat this program loop indefinetly so loang as it is powered.

 Note: The arduinoFFT.h library needs to be added to the Arduino IDE before compiling and uploading this script/sketch to an Arduino.

 License: This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License (GPL) version 3, or any later
 version of your choice, as published by the Free Software Foundation.

 Notes: Copyright (c) 2019 by C. A. Lettsome Services, LLC
 For more information visit https://clydelettsome.com/blog/2019/12/18/my-weekend-project-audio-frequency-detector-using-an-arduino/

*/

#include "arduinoFFT.h"
#include <FastLED.h>

#define LED_PIN     2
#define NUM_LEDS    64
#define SAMPLES 128             //SAMPLES-pt FFT. Must be a base 2 number. Max 128 for Arduino Uno.
#define SAMPLING_FREQUENCY 2048 //Ts = Based on Nyquist, must be 2 times the highest expected frequency.


arduinoFFT FFT = arduinoFFT();
CRGB leds[NUM_LEDS];
 
unsigned int samplingPeriod;
unsigned long microSeconds;
unsigned int num;
 
double vReal[SAMPLES]; //create vector of size SAMPLES to hold real values
double vImag[SAMPLES]; //create vector of size SAMPLES to hold imaginary values
 
void setup() 
{
    Serial.begin(115200); //Baud rate for the Serial Monitor
    samplingPeriod = round(1000000*(1.0/SAMPLING_FREQUENCY)); //Period in microseconds
    FastLED.addLeds<WS2812, LED_PIN, GRB>(leds, NUM_LEDS); 
}
 
void loop() 
{  
    /*Sample SAMPLES times*/
    for(int i=0; i<SAMPLES; i++)
    {
        microSeconds = micros();    //Returns the number of microseconds since the Arduino board began running the current script. 
     
        vReal[i] = analogRead(0); //Reads the value from analog pin 0 (A0), quantize it and save it as a real term.
        vImag[i] = 0; //Makes imaginary term 0 always

        /*remaining wait time between samples if necessary*/
        while(micros() < (microSeconds + samplingPeriod))
        {
          //do nothing
        }
    }
 
    /*Perform FFT on samples*/
    FFT.Windowing(vReal, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
    FFT.Compute(vReal, vImag, SAMPLES, FFT_FORWARD);
    FFT.ComplexToMagnitude(vReal, vImag, SAMPLES);

    /*Find peak frequency and print peak*/
    double peak = FFT.MajorPeak(vReal, SAMPLES, SAMPLING_FREQUENCY);
    Serial.println(peak);     //Print out the most dominant frequency.

  if (peak>100 && peak<150){
    num = 0;
  }

  if (peak>150 && peak<200){
    num = 1;
  }

  if (peak>200 && peak<250){
    num = 2;
  }

  if (peak>250 && peak<300){
    num = 3;
  }

  if (peak>300 && peak<350){
    num = 4;
  }

  if (peak>350 && peak<400){
    num = 5;
  }

  if (peak>400 && peak<450){
    num = 6;
  }

  if (peak>450 && peak<500){
    num = 7;
  }

  if (peak>500 && peak<550){
    num = 8;
  }

  if (peak>550 && peak<600){
    num = 9;
  }

  if (peak>600 && peak<650){
    num = 10;
  }

  if (peak>650 && peak<700){
    num = 11;
  }

  if (peak>700 && peak<750){
    num = 12;
  }

  if (peak>750 && peak<800){
    num = 13;
  }

  if (peak>800 && peak<850){
    num = 14;
  }

  if (peak>850 && peak<900){
    num = 15;
  }

  if (peak>900 && peak<950){
    num = 16;
  }

  if (peak>950 && peak<1000){
    num = 17;
  }

  Serial.println(num);

  switch (num){
  case 0:
   for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 0, 0, 255);
      FastLED.show();
      delay(5);
    }
    break;
    
   case 1:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 0, 255, 0);
      FastLED.show();
      delay(5);
    }
    break;

   case 2:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 234, 10, 132);
      FastLED.show();
      delay(5);
    }
    break;
   
   case 3:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 21, 244, 238);
      FastLED.show();
      delay(5);
    }
    break;

   case 4:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 250, 128, 114);
      FastLED.show();
      delay(5);
    }
    break;

   case 5:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 106, 113, 173);
      FastLED.show();
      delay(5);
    }
    break;

   case 6:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 255, 0, 0);
      FastLED.show();
      delay(5);
    }
    break;

   case 7:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 195, 192, 20);
      FastLED.show();
      delay(5);
    }
    break;

   case 8:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 255, 140, 0);
      FastLED.show();
      delay(5);
    }
    break;

   case 9:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 215, 252, 0);
      FastLED.show();
      delay(5);
    }
    break;

   case 10:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 100, 35, 86);
      FastLED.show();
      delay(5);
    }
    break;

   case 11:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 145, 175, 237);
      FastLED.show();
      delay(5);
    }
    break;

   case 12:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 175, 237, 204);
      FastLED.show();
      delay(5);
    }
    break;

   case 13:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 0, 0, 255);
      FastLED.show();
      delay(5);
    }
    break;

   case 14:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 48, 64, 69);
      FastLED.show();
      delay(5);
    }
    break;

   case 15:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 69, 169, 69);
      FastLED.show();
      delay(5);
    }
    break;

   case 16:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 237, 120, 9);
      FastLED.show();
      delay(5);
    }
    break;

   case 17:
    for (int i = 0; i <= 64; i++) {
      leds[i] = CRGB ( 13, 100, 255);
      FastLED.show();
      delay(5);
    }
   break;

   delay(50);
    /*Script stops here. Hardware reset required.*/
  }
}
